# -*- coding: utf-8 -*-
from abjad.tools import abctools
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools import mathtools
from abjad.tools import pitchtools
from abjad.tools import scoretools
from abjad.tools import sequencetools
from abjad.tools import spannertools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import select


class ReducedLyParser(abctools.Parser):
    r'''Parses the "reduced-ly" syntax, a modified subset of LilyPond syntax.

        >>> parser = lilypondparsertools.ReducedLyParser()

    Understands LilyPond-like representation of notes, chords and rests:

    ::

        >>> string = "c'4 r8. <b d' fs'>16"
        >>> result = parser(string)
        >>> print(format(result))
        {
            c'4
            r8.
            <b d' fs'>16
        }

    Also parses bare duration as notes on middle-C, and negative bare
    durations as rests:

    ::

        >>> string = '4 -8 16. -32'
        >>> result = parser(string)
        >>> print(format(result))
        {
            c'4
            r8
            c'16.
            r32
        }

    Note that the leaf syntax is greedy, and therefore duration specifiers
    following pitch specifiers will be treated as part of the same expression.
    The following produces 2 leaves, rather than 3:

    ::

        >>> string = "4 d' 4"
        >>> result = parser(string)
        >>> print(format(result))
        {
            c'4
            d'4
        }

    Understands LilyPond-like default durations:

    ::

        >>> string = "c'4 d' e' f'"
        >>> result = parser(string)
        >>> print(format(result))
        {
            c'4
            d'4
            e'4
            f'4
        }

    Also understands various types of container specifications.

    Can create arbitrarily nested tuplets:

    ::

        >>> string = "2/3 { 4 4 3/5 { 8 8 8 } }"
        >>> result = parser(string)
        >>> print(format(result))
        \tweak edge-height #'(0.7 . 0)
        \times 2/3 {
            c'4
            c'4
            \tweak text #tuplet-number::calc-fraction-text
            \tweak edge-height #'(0.7 . 0)
            \times 3/5 {
                c'8
                c'8
                c'8
            }
        }

    Can also create empty `FixedDurationContainers`:

    ::

        >>> string = '{1/4} {3/4}'
        >>> result = parser(string)
        >>> for x in result: x
        ...
        FixedDurationContainer(Duration(1, 4), [])
        FixedDurationContainer(Duration(3, 4), [])

    Can create measures too:

    ::

        >>> string = '| 4/4 4 4 4 4 || 3/8 8 8 8 |'
        >>> result = parser(string)
        >>> for x in result: x
        ...
        Measure((4, 4), "c'4 c'4 c'4 c'4")
        Measure((3, 8), "c'8 c'8 c'8")

    Finally, understands ties, slurs and beams:

    ::

        >>> string = 'c16 [ ( d ~ d ) f ]'
        >>> result = parser(string)
        >>> print(format(result))
        {
            c16 [ (
            d16 ~
            d16 )
            f16 ]
        }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_duration',
        '_toplevel_component_count',
        )

    ### INITIALIZER ###

    def __init__(self, debug=False):
        self._default_duration = durationtools.Duration((1, 4))
        self._toplevel_component_count = None
        abctools.Parser.__init__(self, debug=debug)

    ### LEX SETUP ###

    tokens = (
        'APOSTROPHE',
        'BRACE_L',
        'BRACE_R',
        'BRACKET_L',
        'BRACKET_R',
        'CARAT_L',
        'CARAT_R',
        'COMMA',
        'DOT',
        'FRACTION',
        'INTEGER_N',
        'INTEGER_P',
        'PAREN_L',
        'PAREN_R',
        'PIPE',
        'PITCHNAME',
        'RESTNAME',
        'TILDE',
    )

    t_APOSTROPHE = "'"
    t_BRACE_L = '{'
    t_BRACE_R = '}'
    t_BRACKET_L = '\['
    t_BRACKET_R = '\]'
    t_CARAT_L = '\<'
    t_CARAT_R = '\>'
    t_COMMA = ','
    t_DOT = '\.'
    t_PAREN_L = '\('
    t_PAREN_R = '\)'
    t_PIPE = '\|'
    t_RESTNAME = 'r'
    t_TILDE = '~'

    t_ignore = ' \t\r'

    ### YACC SETUP ###

    start = 'start'

    ### LEX METHODS ###

    def t_FRACTION(self, t):
        r'([1-9]\d*/[1-9]\d*)'
        parts = t.value.split('/')
        t.value = mathtools.NonreducedFraction(int(parts[0]), int(parts[1]))
        return t

    def t_INTEGER_N(self, t):
        r'(-[1-9]\d*)'
        t.value = int(t.value)
        return t

    def t_INTEGER_P(self, t):
        r'([1-9]\d*)'
        t.value = int(t.value)
        return t

    def t_PITCHNAME(self, t):
        r'[a-g](ff|ss|f|s|tqf|tqs|qs|qf)?'
        t.value = pitchtools.NamedPitchClass(t.value)
        return t

    def t_error(self, t):
        print(("Illegal character '%s'" % t.value[0]))
        t.lexer.skip(1)

    def t_newline(self, t):
        r'\n+'
        t.lexer.lineno += t.value.count('\n')

    ### YACC METHODS ###

    def p_apostrophes__APOSTROPHE(self, p):
        r'''apostrophes : APOSTROPHE
        '''
        p[0] = 1

    def p_apostrophes__apostrophes__APOSTROPHE(self, p):
        r'''apostrophes : apostrophes APOSTROPHE
        '''
        p[0] = p[1] + 1

    def p_beam__BRACKET_L(self, p):
        r'''beam : BRACKET_L
        '''
        p[0] = (spannertools.Beam, Left)

    def p_beam__BRACKET_R(self, p):
        r'''beam : BRACKET_R
        '''
        p[0] = (spannertools.Beam, Right)

    def p_chord_body__chord_pitches(self, p):
        r'''chord_body : chord_pitches
        '''
        p[0] = scoretools.Chord(p[1], self._default_duration)

    def p_chord_body__chord_pitches__positive_leaf_duration(self, p):
        r'''chord_body : chord_pitches positive_leaf_duration
        '''
        p[0] = scoretools.Chord(p[1], p[2])

    def p_chord_pitches__CARAT_L__pitches__CARAT_R(self, p):
        r'''chord_pitches : CARAT_L pitches CARAT_R
        '''
        p[0] = p[2]

    def p_commas__COMMA(self, p):
        r'''commas : COMMA
        '''
        p[0] = 1

    def p_commas__commas__commas(self, p):
        r'''commas : commas COMMA
        '''
        p[0] = p[1] + 1

    def p_component__container(self, p):
        r'''component : container
        '''
        p[0] = p[1]

    def p_component__fixed_duration_container(self, p):
        r'''component : fixed_duration_container
        '''
        p[0] = p[1]

    def p_component__leaf(self, p):
        r'''component : leaf
        '''
        p[0] = p[1]

    def p_component__tuplet(self, p):
        r'''component : tuplet
        '''
        p[0] = p[1]

    def p_component_list__EMPTY(self, p):
        r'''component_list :
        '''
        p[0] = []

    def p_component_list__component_list__component(self, p):
        r'''component_list : component_list component
        '''
        p[0] = p[1] + [p[2]]

    def p_container__BRACE_L__component_list__BRACE_R(self, p):
        r'''container : BRACE_L component_list BRACE_R
        '''
        p[0] = scoretools.Container()
        for component in p[2]:
            p[0].append(component)

    def p_dots__EMPTY(self, p):
        r'''dots :
        '''
        p[0] = 0

    def p_dots__dots__DOT(self, p):
        r'''dots : dots DOT
        '''
        p[0] = p[1] + 1

    def p_error(self, p):
        if p:
            print(("Syntax error at '%s'" % p.value))
        else:
            print("Syntax error at EOF")

    def p_fixed_duration_container__BRACE_L__FRACTION__BRACE_R(self, p):
        r'''fixed_duration_container : BRACE_L FRACTION BRACE_R
        '''
        p[0] = scoretools.FixedDurationContainer(durationtools.Duration(p[2]))

    def p_leaf__leaf_body__post_events(self, p):
        r'''leaf : leaf_body post_events
        '''
        p[0] = p[1]
        if p[2]:
            annotation = indicatortools.Annotation('post events', p[2])
            attach(annotation, p[0])

    def p_leaf_body__chord_body(self, p):
        r'''leaf_body : chord_body
        '''
        p[0] = p[1]

    def p_leaf_body__note_body(self, p):
        r'''leaf_body : note_body
        '''
        p[0] = p[1]

    def p_leaf_body__rest_body(self, p):
        r'''leaf_body : rest_body
        '''
        p[0] = p[1]

    def p_measure__PIPE__FRACTION__component_list__PIPE(self, p):
        r'''measure : PIPE FRACTION component_list PIPE
        '''
        measure = scoretools.Measure(p[2].pair)
        for x in p[3]:
            measure.append(x)
        p[0] = measure

    def p_negative_leaf_duration__INTEGER_N__dots(self, p):
        r'''negative_leaf_duration : INTEGER_N dots
        '''
        duration_log = p[1]
        dots = '.' * p[2]
        duration = durationtools.Duration.from_lilypond_duration_string(
            '{}{}'.format(abs(duration_log), dots))
        self._default_duration = duration
        p[0] = duration

    def p_note_body__pitch(self, p):
        r'''note_body : pitch
        '''
        p[0] = scoretools.Note(p[1], self._default_duration)

    def p_note_body__pitch__positive_leaf_duration(self, p):
        r'''note_body : pitch positive_leaf_duration
        '''
        p[0] = scoretools.Note(p[1], p[2])

    def p_note_body__positive_leaf_duration(self, p):
        r'''note_body : positive_leaf_duration
        '''
        p[0] = scoretools.Note(0, p[1])

    def p_pitch__PITCHNAME(self, p):
        r'''pitch : PITCHNAME
        '''
        p[0] = pitchtools.NamedPitch(str(p[1]))

    def p_pitch__PITCHNAME__apostrophes(self, p):
        r'''pitch : PITCHNAME apostrophes
        '''
        p[0] = pitchtools.NamedPitch(str(p[1]) + "'" * p[2])

    def p_pitch__PITCHNAME__commas(self, p):
        r'''pitch : PITCHNAME commas
        '''
        p[0] = pitchtools.NamedPitch(str(p[1]) + ',' * p[2])

    def p_pitches__pitch(self, p):
        r'''pitches : pitch
        '''
        p[0] = [p[1]]

    def p_pitches__pitches__pitch(self, p):
        r'''pitches : pitches pitch
        '''
        p[0] = p[1] + [p[2]]

    def p_positive_leaf_duration__INTEGER_P__dots(self, p):
        r'''positive_leaf_duration : INTEGER_P dots
        '''
        duration_log = p[1]
        dots = '.' * p[2]
        duration = durationtools.Duration.from_lilypond_duration_string(
            '{}{}'.format(abs(duration_log), dots))
        self._default_duration = duration
        p[0] = duration

    def p_post_event__beam(self, p):
        r'''post_event : beam
        '''
        p[0] = p[1]

    def p_post_event__slur(self, p):
        r'''post_event : slur
        '''
        p[0] = p[1]

    def p_post_event__tie(self, p):
        r'''post_event : tie
        '''
        p[0] = p[1]

    def p_post_events__EMPTY(self, p):
        r'''post_events :
        '''
        p[0] = {}

    def p_post_events__post_events__post_event(self, p):
        r'''post_events : post_events post_event
        '''
        kind, direction = p[2]
        if kind in p[1]:
            p[1][kind].append(direction)
        else:
            p[1][kind] = [direction]
        p[0] = p[1]

    def p_rest_body__RESTNAME(self, p):
        r'''rest_body : RESTNAME
        '''
        p[0] = scoretools.Rest(self._default_duration)

    def p_rest_body__RESTNAME__positive_leaf_duration(self, p):
        r'''rest_body : RESTNAME positive_leaf_duration
        '''
        p[0] = scoretools.Rest(p[2])

    def p_rest_body__negative_leaf_duration(self, p):
        r'''rest_body : negative_leaf_duration
        '''
        p[0] = scoretools.Rest(p[1])

    def p_slur__PAREN_L(self, p):
        r'''slur : PAREN_L
        '''
        p[0] = (spannertools.Slur, Left)

    def p_slur__PAREN_R(self, p):
        r'''slur : PAREN_R
        '''
        p[0] = (spannertools.Slur, Right)

    def p_start__EMPTY(self, p):
        r'''start :
        '''
        self._toplevel_component_count = 0
        p[0] = []

    def p_start__start__component(self, p):
        r'''start : start component
        '''
        self._toplevel_component_count += 1
        p[0] = p[1] + [p[2]]

    def p_start__start__measure(self, p):
        r'''start : start measure
        '''
        self._toplevel_component_count += 1
        p[0] = p[1] + [p[2]]

    def p_tie__TILDE(self, p):
        r'''tie : TILDE
        '''
        p[0] = (spannertools.Tie, Left)

    def p_tuplet__FRACTION__container(self, p):
        r'''tuplet : FRACTION container
        '''
        p[0] = scoretools.Tuplet(p[1], p[2][:])

    ### PRIVATE METHODS ###

    def _apply_spanners(self, leaves):

        spanner_references = {
            spannertools.Beam: None,
            spannertools.Slur: None,
        }

        first_leaf = leaves[0]
        pairs = sequencetools.Sequence(leaves).nwise(wrapped=True)
        for leaf, next_leaf in pairs:
            span_events = self._get_span_events(leaf)
            for current_class, directions in span_events.items():
                starting, stopping = [], []
                for direction in directions:
                    if direction == Left:
                        starting.append(Left)
                    else:
                        stopping.append(Right)

                # apply undirected events immediately,
                # and do not maintain a reference to them
                if current_class is spannertools.Tie:
                    if next_leaf is first_leaf:
                        message = 'unterminated {} at {}.'
                        message = message.format(current_class.__name__, leaf)
                        raise Exception(message)
                    previous_tie = [
                        x for x in leaf._get_spanners()
                        if isinstance(x, spannertools.Tie)
                        ]
                    if previous_tie:
                        previous_tie[0]._append(next_leaf)
                    else:
                        tie = spannertools.Tie()
                        attach(tie, (leaf, next_leaf))

                elif current_class is spannertools.Beam:
                    # A beam may begin and end on the same leaf
                    # but only one beam spanner may cover any given leaf,
                    # and starting events are processed before ending ones
                    for _ in starting:
                        if spanner_references[current_class] is not None:
                            message = 'already have beam.'
                            raise Exception(message)
                        else:
                            spanner_references[current_class] = current_class()
                    for _ in stopping:
                        if spanner_references[current_class] is not None:
                            spanner_references[current_class]._append(leaf)
                            spanner_references[current_class] = None

                elif current_class is spannertools.Slur:
                    # Slurs process stop events before start events,
                    # they must contain more than one leaf,
                    # but they can stop on a leaf and start on the same leaf.
                    for _ in stopping:
                        if spanner_references[current_class] is not None:
                            spanner_references[current_class]._append(leaf)
                            spanner_references[current_class] = None
                        else:
                            message = 'can not end: {}.'
                            message = message.format(current_class.__name)
                            raise Exception(message)
                    for _ in starting:
                        if spanner_references[current_class] is None:
                            spanner_references[current_class] = current_class()
                        else:
                            message = 'already have: {}.'
                            message = message.format(current_class.__name)
                            raise Exception(message)

            # append leaf to all tracked spanners,
            for current_class, instance in spanner_references.items():
                if instance is not None:
                    instance._append(leaf)

        # check for unterminated spanners
        for current_class, instance in spanner_references.items():
            if instance is not None:
                message = 'unterminated {}.'
                message = message.format(current_class.__name__)
                raise Exception(message)

    def _cleanup(self, parsed):
        container = scoretools.Container()
        for x in parsed:
            container.append(x)
        parsed = container
        leaves = select(parsed).by_leaf()
        if leaves:
            self._apply_spanners(leaves)
        for leaf in leaves:
            detach(indicatortools.Annotation, leaf)
        if 1 < self._toplevel_component_count:
            return parsed
        return parsed[0]

    def _get_span_events(self, leaf):
        annotations = leaf._get_indicators(indicatortools.Annotation)
        detach(indicatortools.Annotation, leaf)
        annotations = [x for x in annotations if x.name == 'post events']
        if annotations:
            return annotations[0].value
        return {}

    def _setup(self):
        self._toplevel_component_count = 0
        self._default_duration = durationtools.Duration((1, 4))

    ### PUBLIC PROPERTIES ###

    @property
    def debug(self):
        r'''Gets debug boolean of reduced ly parser.

        Returns true or false.
        '''
        return self._debug

    @property
    def lexer_rules_object(self):
        r'''Lexer rules object of reduced ly parser.
        '''
        return self

    @property
    def parser_rules_object(self):
        r'''Parser rules object of reduced ly parser.
        '''
        return self



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedPitchClass_diatonic_pitch_class_number_01():

    assert pitchtools.NumberedPitchClass(0).diatonic_pitch_class_number == 0
    assert pitchtools.NumberedPitchClass(1).diatonic_pitch_class_number == 0
    assert pitchtools.NumberedPitchClass(2).diatonic_pitch_class_number == 1
    assert pitchtools.NumberedPitchClass(3).diatonic_pitch_class_number == 2
    assert pitchtools.NumberedPitchClass(4).diatonic_pitch_class_number == 2
    assert pitchtools.NumberedPitchClass(5).diatonic_pitch_class_number == 3
    assert pitchtools.NumberedPitchClass(6).diatonic_pitch_class_number == 3
    assert pitchtools.NumberedPitchClass(7).diatonic_pitch_class_number == 4
    assert pitchtools.NumberedPitchClass(8).diatonic_pitch_class_number == 5
    assert pitchtools.NumberedPitchClass(9).diatonic_pitch_class_number == 5
    assert pitchtools.NumberedPitchClass(10).diatonic_pitch_class_number == 6
    assert pitchtools.NumberedPitchClass(11).diatonic_pitch_class_number == 6



# -*- coding: utf-8 -*-
from abjad.tools import commandlinetools
from abjad.tools import systemtools
from base import ScorePackageScriptTestCase


class Test(ScorePackageScriptTestCase):

    def test_list(self):
        self.create_score()
        script = commandlinetools.ManageBuildTargetScript()
        command = ['--new', 'big-version']
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            try:
                script(command)
            except SystemExit:
                raise RuntimeError('SystemExit')
        command = ['--new', 'medium-version']
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            try:
                script(command)
            except SystemExit:
                raise RuntimeError('SystemExit')
        command = ['--new', 'small-version']
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            try:
                script(command)
            except SystemExit:
                raise RuntimeError('SystemExit')
        command = ['--list']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                try:
                    script(command)
                except SystemExit:
                    raise RuntimeError('SystemExit')
        self.compare_captured_output(r'''
        Available build targets:
            big-version
            medium-version
            small-version
        ''')



# -*- coding: utf-8 -*-
from abjad.tools.indicatortools.LineSegment import LineSegment


class Arrow(LineSegment):
    r'''Arrow.

    ..  container:: example

        **Example 1.** String contact position spanner:
            
        ::

            >>> staff = Staff("c'4 d'4 e'4 f'4")
            >>> start_markup = Markup('pont.').upright()
            >>> stop_markup = Markup('ord.').upright()
            >>> arrow = indicatortools.Arrow()

        ::

            >>> print(format(arrow))
            indicatortools.Arrow(
                arrow_width=0.25,
                dash_fraction=1,
                left_broken_text=False,
                left_hspace=0.25,
                left_stencil_align_direction_y=Center,
                right_arrow=True,
                right_broken_padding=0,
                right_padding=1.5,
                right_stencil_align_direction_y=Center,
                )

        ::

            >>> attach(start_markup, staff[0], is_annotation=True)
            >>> attach(stop_markup, staff[-1], is_annotation=True)
            >>> attach(arrow, staff[0])
            >>> attach(spannertools.TextSpanner(), staff[:])

        ::

            >>> override(staff).text_script.staff_padding = 1.25
            >>> override(staff).text_spanner.staff_padding = 2
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                \override TextScript.staff-padding = #1.25
                \override TextSpanner.staff-padding = #2
            } {
                \once \override TextSpanner.arrow-width = 0.25
                \once \override TextSpanner.bound-details.left-broken.text = ##f
                \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                \once \override TextSpanner.bound-details.left.text = \markup {
                    \concat
                        {
                            \upright
                                pont.
                            \hspace
                                #0.25
                        }
                    }
                \once \override TextSpanner.bound-details.right-broken.padding = 0
                \once \override TextSpanner.bound-details.right.arrow = ##t
                \once \override TextSpanner.bound-details.right.padding = 1.5
                \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                \once \override TextSpanner.dash-fraction = 1
                c'4 \startTextSpan
                d'4
                e'4
                f'4 \stopTextSpan ^ \markup {
                    \upright
                        ord.
                    }
            }

    Arrow is a preconfigured line segment.

    Arrow formats as a text spanner.

    Follow the piecewise definition protocol shown here.
    '''

    ### CLASS VARIABLES ###

    # must be present for copy, deepcopy, pickle
    __slots__ = (
        '_default_scope',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        arrow_width=0.25,
        dash_fraction=1,
        dash_period=None,
        left_broken_padding=None,
        left_broken_text=False,
        left_hspace=0.25,
        left_padding=None,
        left_stencil_align_direction_y=Center,
        right_arrow=True,
        right_broken_arrow=None,
        right_broken_padding=0,
        right_padding=1.5,
        right_stencil_align_direction_y=Center,
        style=None,
        ):
        superclass = super(Arrow, self)
        self._default_scope = None
        superclass.__init__(
            arrow_width=arrow_width,
            dash_fraction=dash_fraction,
            dash_period=dash_period,
            left_broken_padding=left_broken_padding,
            left_broken_text=left_broken_text,
            left_hspace=left_hspace,
            left_padding=left_padding,
            left_stencil_align_direction_y=left_stencil_align_direction_y,
            right_arrow=right_arrow,
            right_broken_arrow=right_broken_arrow,
            right_broken_padding=right_broken_padding,
            right_padding=right_padding,
            right_stencil_align_direction_y=right_stencil_align_direction_y,
            style=style,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def arrow_width(self):
        r'''Gets arrow width of arrow.
        
        ..  container:: example

            **Example 1.** Arrow width equals ``0.25``:

            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> start_markup = Markup('pont.').upright()
                >>> stop_markup = Markup('ord.').upright()
                >>> arrow = indicatortools.Arrow(arrow_width=0.25)

            ::

                >>> attach(start_markup, staff[0], is_annotation=True)
                >>> attach(stop_markup, staff[-1], is_annotation=True)
                >>> attach(arrow, staff[0])
                >>> attach(spannertools.TextSpanner(), staff[:])

            ::

                >>> override(staff).text_script.staff_padding = 1.25
                >>> override(staff).text_spanner.staff_padding = 2
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #1.25
                    \override TextSpanner.staff-padding = #2
                } {
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.text = ##f
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.25
                            }
                        }
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 1.5
                    \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                    \once \override TextSpanner.dash-fraction = 1
                    c'4 \startTextSpan
                    d'4
                    e'4
                    f'4 \stopTextSpan ^ \markup {
                        \upright
                            ord.
                        }
                }

            Results in thin arrow head.

            This is default behavior.

        ..  container:: example

            **Example 2.** Arrow width equals ``0.5``:

            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> start_markup = Markup('pont.').upright()
                >>> stop_markup = Markup('ord.').upright()
                >>> arrow = indicatortools.Arrow(arrow_width=0.5)

            ::

                >>> attach(start_markup, staff[0], is_annotation=True)
                >>> attach(stop_markup, staff[-1], is_annotation=True)
                >>> attach(arrow, staff[0])
                >>> attach(spannertools.TextSpanner(), staff[:])

            ::

                >>> override(staff).text_script.staff_padding = 1.25
                >>> override(staff).text_spanner.staff_padding = 2
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #1.25
                    \override TextSpanner.staff-padding = #2
                } {
                    \once \override TextSpanner.arrow-width = 0.5
                    \once \override TextSpanner.bound-details.left-broken.text = ##f
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.25
                            }
                        }
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 1.5
                    \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                    \once \override TextSpanner.dash-fraction = 1
                    c'4 \startTextSpan
                    d'4
                    e'4
                    f'4 \stopTextSpan ^ \markup {
                        \upright
                            ord.
                        }
                }

            Results in wide arrow head.

        ..  container:: example

            **Example 3.** Arrow width equals ``1``:

            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> start_markup = Markup('pont.').upright()
                >>> stop_markup = Markup('ord.').upright()
                >>> arrow = indicatortools.Arrow(arrow_width=1)

            ::

                >>> attach(start_markup, staff[0], is_annotation=True)
                >>> attach(stop_markup, staff[-1], is_annotation=True)
                >>> attach(arrow, staff[0])
                >>> attach(spannertools.TextSpanner(), staff[:])

            ::

                >>> override(staff).text_script.staff_padding = 1.25
                >>> override(staff).text_spanner.staff_padding = 2
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #1.25
                    \override TextSpanner.staff-padding = #2
                } {
                    \once \override TextSpanner.arrow-width = 1
                    \once \override TextSpanner.bound-details.left-broken.text = ##f
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.25
                            }
                        }
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 1.5
                    \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                    \once \override TextSpanner.dash-fraction = 1
                    c'4 \startTextSpan
                    d'4
                    e'4
                    f'4 \stopTextSpan ^ \markup {
                        \upright
                            ord.
                        }
                }

            Results in very wide arrow head.

        Defaults to ``0.25``.

        Returns integer or float.
        '''
        superclass = super(Arrow, self)
        return superclass.arrow_width

    @property
    def dash_fraction(self):
        r'''Gets dash fraction of arrow.
        
        ..  container:: example

            **Example 1.** Dash fraction equals 100% of dash period:


            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> start_markup = Markup('pont.').upright()
                >>> stop_markup = Markup('ord.').upright()
                >>> arrow = indicatortools.Arrow(dash_fraction=1)

            ::

                >>> attach(start_markup, staff[0], is_annotation=True)
                >>> attach(stop_markup, staff[-1], is_annotation=True)
                >>> attach(arrow, staff[0])
                >>> attach(spannertools.TextSpanner(), staff[:])

            ::

                >>> override(staff).text_script.staff_padding = 1.25
                >>> override(staff).text_spanner.staff_padding = 2
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #1.25
                    \override TextSpanner.staff-padding = #2
                } {
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.text = ##f
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.25
                            }
                        }
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 1.5
                    \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                    \once \override TextSpanner.dash-fraction = 1
                    c'4 \startTextSpan
                    d'4
                    e'4
                    f'4 \stopTextSpan ^ \markup {
                        \upright
                            ord.
                        }
                }

            This is default behavior.

        ..  container:: example

            **Example 2.** Dash fraction equals 50% of dash period:


            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> start_markup = Markup('pont.').upright()
                >>> stop_markup = Markup('ord.').upright()
                >>> arrow = indicatortools.Arrow(dash_fraction=0.5)

            ::

                >>> attach(start_markup, staff[0], is_annotation=True)
                >>> attach(stop_markup, staff[-1], is_annotation=True)
                >>> attach(arrow, staff[0])
                >>> attach(spannertools.TextSpanner(), staff[:])

            ::

                >>> override(staff).text_script.staff_padding = 1.25
                >>> override(staff).text_spanner.staff_padding = 2
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #1.25
                    \override TextSpanner.staff-padding = #2
                } {
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.text = ##f
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.25
                            }
                        }
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 1.5
                    \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                    \once \override TextSpanner.dash-fraction = 0.5
                    c'4 \startTextSpan
                    d'4
                    e'4
                    f'4 \stopTextSpan ^ \markup {
                        \upright
                            ord.
                        }
                }

        ..  container:: example

            **Example 3.** Dash fraction equals 10% of dash period:


            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> start_markup = Markup('pont.').upright()
                >>> stop_markup = Markup('ord.').upright()
                >>> arrow = indicatortools.Arrow(dash_fraction=0.1)

            ::

                >>> attach(start_markup, staff[0], is_annotation=True)
                >>> attach(stop_markup, staff[-1], is_annotation=True)
                >>> attach(arrow, staff[0])
                >>> attach(spannertools.TextSpanner(), staff[:])

            ::

                >>> override(staff).text_script.staff_padding = 1.25
                >>> override(staff).text_spanner.staff_padding = 2
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #1.25
                    \override TextSpanner.staff-padding = #2
                } {
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.text = ##f
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.25
                            }
                        }
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 1.5
                    \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                    \once \override TextSpanner.dash-fraction = 0.1
                    c'4 \startTextSpan
                    d'4
                    e'4
                    f'4 \stopTextSpan ^ \markup {
                        \upright
                            ord.
                        }
                }

        Defaults to ``1``.

        Returns integer or float.
        '''
        superclass = super(Arrow, self)
        return superclass.dash_fraction

    @property
    def dash_period(self):
        r'''Gets dash period of arrow.

        ..  container:: example

            **Example 1.** Dash period equals none:

            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> start_markup = Markup('pont.').upright()
                >>> stop_markup = Markup('ord.').upright()
                >>> arrow = indicatortools.Arrow(
                ...     dash_period=None,
                ...     )

            ::

                >>> attach(start_markup, staff[0], is_annotation=True)
                >>> attach(stop_markup, staff[-1], is_annotation=True)
                >>> attach(arrow, staff[0])
                >>> attach(spannertools.TextSpanner(), staff[:])

            ::

                >>> override(staff).text_script.staff_padding = 1.25
                >>> override(staff).text_spanner.staff_padding = 2
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #1.25
                    \override TextSpanner.staff-padding = #2
                } {
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.text = ##f
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.25
                            }
                        }
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 1.5
                    \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                    \once \override TextSpanner.dash-fraction = 1
                    c'4 \startTextSpan
                    d'4
                    e'4
                    f'4 \stopTextSpan ^ \markup {
                        \upright
                            ord.
                        }
                }

            Results in a solid line.

            This is default behavior.

        ..  container:: example

            **Example 2.** Dash period equals ``1`` (with dash fraction equal
            to 25%):

            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> start_markup = Markup('pont.').upright()
                >>> stop_markup = Markup('ord.').upright()
                >>> arrow = indicatortools.Arrow(
                ...     dash_fraction=0.25,
                ...     dash_period=1,
                ...     )

            ::

                >>> attach(start_markup, staff[0], is_annotation=True)
                >>> attach(stop_markup, staff[-1], is_annotation=True)
                >>> attach(arrow, staff[0])
                >>> attach(spannertools.TextSpanner(), staff[:])

            ::

                >>> override(staff).text_script.staff_padding = 1.25
                >>> override(staff).text_spanner.staff_padding = 2
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #1.25
                    \override TextSpanner.staff-padding = #2
                } {
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.text = ##f
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.25
                            }
                        }
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 1.5
                    \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                    \once \override TextSpanner.dash-fraction = 0.25
                    \once \override TextSpanner.dash-period = 1
                    c'4 \startTextSpan
                    d'4
                    e'4
                    f'4 \stopTextSpan ^ \markup {
                        \upright
                            ord.
                        }
                }

            Results in fine dashes.

        ..  container:: example

            **Example 3.** Dash period equals ``4`` (with dash fraction equal
            to 25%):

            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> start_markup = Markup('pont.').upright()
                >>> stop_markup = Markup('ord.').upright()
                >>> arrow = indicatortools.Arrow(
                ...     dash_fraction=0.25,
                ...     dash_period=4,
                ...     )

            ::

                >>> attach(start_markup, staff[0], is_annotation=True)
                >>> attach(stop_markup, staff[-1], is_annotation=True)
                >>> attach(arrow, staff[0])
                >>> attach(spannertools.TextSpanner(), staff[:])

            ::

                >>> override(staff).text_script.staff_padding = 1.25
                >>> override(staff).text_spanner.staff_padding = 2
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #1.25
                    \override TextSpanner.staff-padding = #2
                } {
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.text = ##f
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.25
                            }
                        }
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 1.5
                    \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                    \once \override TextSpanner.dash-fraction = 0.25
                    \once \override TextSpanner.dash-period = 4
                    c'4 \startTextSpan
                    d'4
                    e'4
                    f'4 \stopTextSpan ^ \markup {
                        \upright
                            ord.
                        }
                }

            Results in coarse dashes.

        Defaults to none.

        Returns integer or float.
        '''
        superclass = super(Arrow, self)
        return superclass.dash_period

    @property
    def default_scope(self):
        r'''Gets default scope of arrow.

        ..  container:: example

            >>> arrow = indicatortools.Arrow()
            >>> arrow.default_scope is None
            True

        Returns none.
        '''
        return self._default_scope

    @property
    def left_broken_text(self):
        r'''Gets left broken text of arrow.

        ..  container:: example

            **Example 1.** Left broken text set to false:

            ..  container:: example

                ::

                    >>> staff = Staff("c'4. d' e' f' g' a' b' c''")
                    >>> attach(TimeSignature((3, 8)), staff)
                    >>> score = Score([staff])
                    >>> command = indicatortools.LilyPondCommand('break', 'after')
                    >>> attach(command, staff[3])

                ::

                    >>> start_markup = Markup('pont.').upright()
                    >>> stop_markup = Markup('ord.').upright()
                    >>> arrow = indicatortools.Arrow()

                ::

                    >>> print(format(arrow))
                    indicatortools.Arrow(
                        arrow_width=0.25,
                        dash_fraction=1,
                        left_broken_text=False,
                        left_hspace=0.25,
                        left_stencil_align_direction_y=Center,
                        right_arrow=True,
                        right_broken_padding=0,
                        right_padding=1.5,
                        right_stencil_align_direction_y=Center,
                        )

                ::

                    >>> attach(start_markup, staff[2], is_annotation=True)
                    >>> attach(stop_markup, staff[6], is_annotation=True)
                    >>> attach(arrow, staff[2])
                    >>> attach(spannertools.TextSpanner(), staff[2:])

                ::

                    >>> override(staff).text_script.staff_padding = 1.25
                    >>> override(staff).text_spanner.staff_padding = 2
                    >>> show(staff) # doctest: +SKIP

                ..  doctest::

                    >>> f(staff)
                    \new Staff \with {
                        \override TextScript.staff-padding = #1.25
                        \override TextSpanner.staff-padding = #2
                    } {
                        \time 3/8
                        c'4.
                        d'4.
                        \once \override TextSpanner.arrow-width = 0.25
                        \once \override TextSpanner.bound-details.left-broken.text = ##f
                        \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                        \once \override TextSpanner.bound-details.left.text = \markup {
                            \concat
                                {
                                    \upright
                                        pont.
                                    \hspace
                                        #0.25
                                }
                            }
                        \once \override TextSpanner.bound-details.right-broken.padding = 0
                        \once \override TextSpanner.bound-details.right.arrow = ##t
                        \once \override TextSpanner.bound-details.right.padding = 1.5
                        \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                        \once \override TextSpanner.dash-fraction = 1
                        e'4. \startTextSpan
                        f'4.
                        \break
                        g'4.
                        a'4.
                        b'4. \stopTextSpan ^ \markup {
                            \upright
                                ord.
                            }
                        c''4.
                    }

            Results in no text immediately after line break.
            (This is default behavior.)

        ..  container:: example

            **Example 2.** Left broken text set explicitly:

            ..  container:: example

                ::

                    >>> staff = Staff("c'4. d' e' f' g' a' b' c''")
                    >>> attach(TimeSignature((3, 8)), staff)
                    >>> score = Score([staff])
                    >>> command = indicatortools.LilyPondCommand('break', 'after')
                    >>> attach(command, staff[3])

                ::

                    >>> start_markup = Markup('pont.').upright()
                    >>> stop_markup = Markup('ord.').upright()
                    >>> left_broken_markup = Markup('(pont./ord.)').upright()
                    >>> arrow = indicatortools.Arrow(
                    ...     left_broken_text=left_broken_markup,
                    ... )

                ::

                    >>> print(format(arrow))
                    indicatortools.Arrow(
                        arrow_width=0.25,
                        dash_fraction=1,
                        left_broken_text=markuptools.Markup(
                            contents=[
                                markuptools.MarkupCommand(
                                    'upright',
                                    '(pont./ord.)'
                                    ),
                                ],
                            ),
                        left_hspace=0.25,
                        left_stencil_align_direction_y=Center,
                        right_arrow=True,
                        right_broken_padding=0,
                        right_padding=1.5,
                        right_stencil_align_direction_y=Center,
                        )

                ::

                    >>> attach(start_markup, staff[2], is_annotation=True)
                    >>> attach(stop_markup, staff[6], is_annotation=True)
                    >>> attach(arrow, staff[2])
                    >>> attach(spannertools.TextSpanner(), staff[2:])

                ::

                    >>> override(staff).text_script.staff_padding = 1.25
                    >>> override(staff).text_spanner.staff_padding = 2
                    >>> show(staff) # doctest: +SKIP

                ..  doctest::

                    >>> f(staff)
                    \new Staff \with {
                        \override TextScript.staff-padding = #1.25
                        \override TextSpanner.staff-padding = #2
                    } {
                        \time 3/8
                        c'4.
                        d'4.
                        \once \override TextSpanner.arrow-width = 0.25
                        \once \override TextSpanner.bound-details.left-broken.text = \markup {
                            \upright
                                (pont./ord.)
                            }
                        \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                        \once \override TextSpanner.bound-details.left.text = \markup {
                            \concat
                                {
                                    \upright
                                        pont.
                                    \hspace
                                        #0.25
                                }
                            }
                        \once \override TextSpanner.bound-details.right-broken.padding = 0
                        \once \override TextSpanner.bound-details.right.arrow = ##t
                        \once \override TextSpanner.bound-details.right.padding = 1.5
                        \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                        \once \override TextSpanner.dash-fraction = 1
                        e'4. \startTextSpan
                        f'4.
                        \break
                        g'4.
                        a'4.
                        b'4. \stopTextSpan ^ \markup {
                            \upright
                                ord.
                            }
                        c''4.
                    }

        Set to markup, boolean or none.
        '''
        return self._left_broken_text

    @property
    def right_broken_arrow(self):
        r'''Is true when arrow should appear immediately before line break.
        Otherwise false.

        ..  container:: example

            **Example 1.** Right broken arrow set to none:

            ..  container:: example

                ::

                    >>> staff = Staff("c'4. d' e' f' g' a' b' c''")
                    >>> attach(TimeSignature((3, 8)), staff)
                    >>> score = Score([staff])
                    >>> command = indicatortools.LilyPondCommand('break', 'after')
                    >>> attach(command, staff[3])

                ::

                    >>> start_markup = Markup('pont.').upright()
                    >>> stop_markup = Markup('ord.').upright()
                    >>> arrow = indicatortools.Arrow()

                ::

                    >>> print(format(arrow))
                    indicatortools.Arrow(
                        arrow_width=0.25,
                        dash_fraction=1,
                        left_broken_text=False,
                        left_hspace=0.25,
                        left_stencil_align_direction_y=Center,
                        right_arrow=True,
                        right_broken_padding=0,
                        right_padding=1.5,
                        right_stencil_align_direction_y=Center,
                        )

                ::

                    >>> attach(start_markup, staff[2], is_annotation=True)
                    >>> attach(stop_markup, staff[6], is_annotation=True)
                    >>> attach(arrow, staff[2])
                    >>> attach(spannertools.TextSpanner(), staff[2:])

                ::

                    >>> override(staff).text_script.staff_padding = 1.25
                    >>> override(staff).text_spanner.staff_padding = 2
                    >>> show(staff) # doctest: +SKIP

                ..  doctest::

                    >>> f(staff)
                    \new Staff \with {
                        \override TextScript.staff-padding = #1.25
                        \override TextSpanner.staff-padding = #2
                    } {
                        \time 3/8
                        c'4.
                        d'4.
                        \once \override TextSpanner.arrow-width = 0.25
                        \once \override TextSpanner.bound-details.left-broken.text = ##f
                        \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                        \once \override TextSpanner.bound-details.left.text = \markup {
                            \concat
                                {
                                    \upright
                                        pont.
                                    \hspace
                                        #0.25
                                }
                            }
                        \once \override TextSpanner.bound-details.right-broken.padding = 0
                        \once \override TextSpanner.bound-details.right.arrow = ##t
                        \once \override TextSpanner.bound-details.right.padding = 1.5
                        \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                        \once \override TextSpanner.dash-fraction = 1
                        e'4. \startTextSpan
                        f'4.
                        \break
                        g'4.
                        a'4.
                        b'4. \stopTextSpan ^ \markup {
                            \upright
                                ord.
                            }
                        c''4.
                    }

            Results in arrow immediately before line break.
            (This is default behavior.)

        ..  container:: example

            **Example 2.** Right broken arrow set to false:

            ..  container:: example

                ::

                    >>> staff = Staff("c'4. d' e' f' g' a' b' c''")
                    >>> attach(TimeSignature((3, 8)), staff)
                    >>> score = Score([staff])
                    >>> command = indicatortools.LilyPondCommand('break', 'after')
                    >>> attach(command, staff[3])

                ::

                    >>> start_markup = Markup('pont.').upright()
                    >>> stop_markup = Markup('ord.').upright()
                    >>> arrow = indicatortools.Arrow(
                    ...     right_broken_arrow=False,
                    ... )

                ::

                    >>> print(format(arrow))
                    indicatortools.Arrow(
                        arrow_width=0.25,
                        dash_fraction=1,
                        left_broken_text=False,
                        left_hspace=0.25,
                        left_stencil_align_direction_y=Center,
                        right_arrow=True,
                        right_broken_arrow=False,
                        right_broken_padding=0,
                        right_padding=1.5,
                        right_stencil_align_direction_y=Center,
                        )

                ::

                    >>> attach(start_markup, staff[2], is_annotation=True)
                    >>> attach(stop_markup, staff[6], is_annotation=True)
                    >>> attach(arrow, staff[2])
                    >>> attach(spannertools.TextSpanner(), staff[2:])

                ::

                    >>> override(staff).text_script.staff_padding = 1.25
                    >>> override(staff).text_spanner.staff_padding = 2
                    >>> show(staff) # doctest: +SKIP

                ..  doctest::

                    >>> f(staff)
                    \new Staff \with {
                        \override TextScript.staff-padding = #1.25
                        \override TextSpanner.staff-padding = #2
                    } {
                        \time 3/8
                        c'4.
                        d'4.
                        \once \override TextSpanner.arrow-width = 0.25
                        \once \override TextSpanner.bound-details.left-broken.text = ##f
                        \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                        \once \override TextSpanner.bound-details.left.text = \markup {
                            \concat
                                {
                                    \upright
                                        pont.
                                    \hspace
                                        #0.25
                                }
                            }
                        \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                        \once \override TextSpanner.bound-details.right-broken.padding = 0
                        \once \override TextSpanner.bound-details.right.arrow = ##t
                        \once \override TextSpanner.bound-details.right.padding = 1.5
                        \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                        \once \override TextSpanner.dash-fraction = 1
                        e'4. \startTextSpan
                        f'4.
                        \break
                        g'4.
                        a'4.
                        b'4. \stopTextSpan ^ \markup {
                            \upright
                                ord.
                            }
                        c''4.
                    }

        Set to true, false or none.
        '''
        return self._right_broken_arrow

    @property
    def style(self):
        r'''Gets style of arrow.

        ..  container:: example

            **Example 1.** Style equals none:

            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> start_markup = Markup('pont.').upright()
                >>> stop_markup = Markup('ord.').upright()
                >>> arrow = indicatortools.Arrow(style=None)

            ::

                >>> attach(start_markup, staff[0], is_annotation=True)
                >>> attach(stop_markup, staff[-1], is_annotation=True)
                >>> attach(arrow, staff[0])
                >>> attach(spannertools.TextSpanner(), staff[:])

            ::

                >>> override(staff).text_script.staff_padding = 1.25
                >>> override(staff).text_spanner.staff_padding = 2
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #1.25
                    \override TextSpanner.staff-padding = #2
                } {
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.text = ##f
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.25
                            }
                        }
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 1.5
                    \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                    \once \override TextSpanner.dash-fraction = 1
                    c'4 \startTextSpan
                    d'4
                    e'4
                    f'4 \stopTextSpan ^ \markup {
                        \upright
                            ord.
                        }
                }

            LilyPond defaults to solid line.

            This is default behavior.

        ..  container:: example

            **Example 2.** Style equals zig-zag:

            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> start_markup = Markup('pont.').upright()
                >>> stop_markup = Markup('ord.').upright()
                >>> arrow = indicatortools.Arrow(style='zigzag')

            ::

                >>> attach(start_markup, staff[0], is_annotation=True)
                >>> attach(stop_markup, staff[-1], is_annotation=True)
                >>> attach(arrow, staff[0])
                >>> attach(spannertools.TextSpanner(), staff[:])

            ::

                >>> override(staff).text_script.staff_padding = 1.25
                >>> override(staff).text_spanner.staff_padding = 2
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #1.25
                    \override TextSpanner.staff-padding = #2
                } {
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.text = ##f
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.25
                            }
                        }
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 1.5
                    \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                    \once \override TextSpanner.dash-fraction = 1
                    \once \override TextSpanner.style = #'zigzag
                    c'4 \startTextSpan
                    d'4
                    e'4
                    f'4 \stopTextSpan ^ \markup {
                        \upright
                            ord.
                        }
                }

        ..  container:: example

            **Example 3.** Style equals trill:

            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> start_markup = Markup('pont.').upright()
                >>> stop_markup = Markup('ord.').upright()
                >>> arrow = indicatortools.Arrow(style='trill')

            ::

                >>> attach(start_markup, staff[0], is_annotation=True)
                >>> attach(stop_markup, staff[-1], is_annotation=True)
                >>> attach(arrow, staff[0])
                >>> attach(spannertools.TextSpanner(), staff[:])

            ::

                >>> override(staff).text_script.staff_padding = 1.25
                >>> override(staff).text_spanner.staff_padding = 2
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #1.25
                    \override TextSpanner.staff-padding = #2
                } {
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.text = ##f
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.25
                            }
                        }
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 1.5
                    \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                    \once \override TextSpanner.dash-fraction = 1
                    \once \override TextSpanner.style = #'trill
                    c'4 \startTextSpan
                    d'4
                    e'4
                    f'4 \stopTextSpan ^ \markup {
                        \upright
                            ord.
                        }
                }

        ..  container:: example

            **Example 4.** Style equals dotted line:

            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> start_markup = Markup('pont.').upright()
                >>> stop_markup = Markup('ord.').upright()
                >>> arrow = indicatortools.Arrow(style='dotted-line')

            ::

                >>> attach(start_markup, staff[0], is_annotation=True)
                >>> attach(stop_markup, staff[-1], is_annotation=True)
                >>> attach(arrow, staff[0])
                >>> attach(spannertools.TextSpanner(), staff[:])

            ::

                >>> override(staff).text_script.staff_padding = 1.25
                >>> override(staff).text_spanner.staff_padding = 2
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #1.25
                    \override TextSpanner.staff-padding = #2
                } {
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.text = ##f
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = #center
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \concat
                            {
                                \upright
                                    pont.
                                \hspace
                                    #0.25
                            }
                        }
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 1.5
                    \once \override TextSpanner.bound-details.right.stencil-align-dir-y = #center
                    \once \override TextSpanner.dash-fraction = 1
                    \once \override TextSpanner.style = #'dotted-line
                    c'4 \startTextSpan
                    d'4
                    e'4
                    f'4 \stopTextSpan ^ \markup {
                        \upright
                            ord.
                        }
                }

        Defaults to none.

        Returns string or none.
        '''
        superclass = super(Arrow, self)
        return superclass.style



# -*- coding: utf-8 -*-
import numbers


def is_integer_equivalent(argument):
    '''Is true when `argument` is an integer-equivalent number. Otherwise
    false.

    ..  container:: example

        ::

            >>> mathtools.is_integer_equivalent(12.0)
            True

        ::

            >>> mathtools.is_integer_equivalent('12')
            True

        ::

            >>> mathtools.is_integer_equivalent('foo')
            False

    Returns true or false.
    '''
    from abjad.tools import mathtools
    if isinstance(argument, numbers.Number):
        return mathtools.is_integer_equivalent_number(argument)
    try:
        int(argument)
        return True
    except (TypeError, ValueError):
        return False



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import selectiontools
from abjad.tools.topleveltools import attach


def make_tied_leaf(
    class_,
    duration,
    decrease_durations_monotonically=True,
    forbidden_written_duration=None,
    pitches=None,
    tie_parts=True,
    use_messiaen_style_ties=False,
    ):
    r'''Makes tied `class_` with `duration`.

    ..  container:: example

        **Example 1.** Make note:

        ::

            >>> leaves = scoretools.make_tied_leaf(
            ...     Note,
            ...     Duration(1, 2),
            ...     pitches='C#5',
            ...     )
            >>> staff = Staff(leaves)
            >>> time_signature = TimeSignature((2, 4))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                \time 2/4
                cs''2
            }

    ..  container:: example

        **Example 2.** Make note and forbid half notes:

        ::

            >>> leaves = scoretools.make_tied_leaf(
            ...     Note,
            ...     Duration(1, 2),
            ...     pitches='C#5',
            ...     forbidden_written_duration=Duration(1, 2),
            ...     )
            >>> staff = Staff(leaves)
            >>> time_signature = TimeSignature((2, 4))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                \time 2/4
                cs''4 ~
                cs''4
            }

    ..  container:: example

        **Example 3.** Make tied note with half notes forbidden and
        durations decreasing monotonically:

        ::

            >>> leaves = scoretools.make_tied_leaf(
            ...     Note,
            ...     Duration(9, 8),
            ...     pitches='C#5',
            ...     forbidden_written_duration=Duration(1, 2),
            ...     decrease_durations_monotonically=True,
            ...     )
            >>> staff = Staff(leaves)
            >>> time_signature = TimeSignature((9, 8))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                \time 9/8
                cs''4 ~
                cs''4 ~
                cs''4 ~
                cs''4 ~
                cs''8
            }

    ..  container:: example

        **Example 4.** Make tied note with half notes forbidden and
        durations increasing monotonically:

        ::

            >>> leaves = scoretools.make_tied_leaf(
            ...     Note,
            ...     Duration(9, 8),
            ...     pitches='C#5',
            ...     forbidden_written_duration=Duration(1, 2),
            ...     decrease_durations_monotonically=False,
            ...     )
            >>> staff = Staff(leaves)
            >>> time_signature = TimeSignature((9, 8))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                \time 9/8
                cs''8 ~
                cs''4 ~
                cs''4 ~
                cs''4 ~
                cs''4
            }

    ..  container:: example

        **Example 5.** Uses Messiaen-style ties:

        ::

            >>> leaves = scoretools.make_tied_leaf(
            ...     Note,
            ...     Duration(9, 8),
            ...     pitches='C#5',
            ...     decrease_durations_monotonically=False,
            ...     forbidden_written_duration=Duration(1, 2),
            ...     use_messiaen_style_ties=True,
            ...     )
            >>> staff = Staff(leaves)
            >>> time_signature = TimeSignature((9, 8))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                \time 9/8
                cs''8
                cs''4 \repeatTie
                cs''4 \repeatTie
                cs''4 \repeatTie
                cs''4 \repeatTie
            }

    Returns selection of leaves.
    '''
    from abjad.tools import scoretools
    from abjad.tools import spannertools

    # check input
    duration = durationtools.Duration(duration)
    if forbidden_written_duration is not None:
        forbidden_written_duration = \
            durationtools.Duration(forbidden_written_duration)
        assert forbidden_written_duration.is_assignable
        assert forbidden_written_duration.numerator == 1

    # find preferred numerator of written durations if necessary
    if (forbidden_written_duration is not None and
        forbidden_written_duration <= duration):
        denominators = [
            2 * forbidden_written_duration.denominator,
            duration.denominator,
            ]
        denominator = mathtools.least_common_multiple(*denominators)
        forbidden_written_duration = \
            mathtools.NonreducedFraction(forbidden_written_duration)
        forbidden_written_duration = \
            forbidden_written_duration.with_denominator(denominator)
        duration = mathtools.NonreducedFraction(duration)
        duration = duration.with_denominator(denominator)
        forbidden_numerator = forbidden_written_duration.numerator
        assert forbidden_numerator % 2 == 0
        preferred_numerator = forbidden_numerator / 2

    # make written duration numerators
    numerators = []
    parts = mathtools.partition_integer_into_canonic_parts(duration.numerator)
    if (forbidden_written_duration is not None and
        forbidden_written_duration <= duration):
        for part in parts:
            if forbidden_numerator <= part:
                better_parts = \
                    mathtools.partition_integer_into_parts_less_than_double(
                    part, preferred_numerator)
                numerators.extend(better_parts)
            else:
                numerators.append(part)
    else:
        numerators = parts

    # reverse numerators if necessary
    if not decrease_durations_monotonically:
        numerators = list(reversed(numerators))

    # make one leaf per written duration
    result = []
    for numerator in numerators:
        written_duration = durationtools.Duration(
            numerator,
            duration.denominator,
            )
        if not pitches is None:
            arguments = (pitches, written_duration)
        else:
            arguments = (written_duration, )
        result.append(class_(*arguments))

    # apply tie spanner if required
    if tie_parts and 1 < len(result):
        if not issubclass(class_, (scoretools.Rest, scoretools.Skip)):
            tie = spannertools.Tie(
                use_messiaen_style_ties=use_messiaen_style_ties,
                )
            attach(tie, result)

    # return result
    result = selectiontools.Selection(result)
    return result



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_agenttools_MutationAgent_split_01():
    r'''Cyclically splits note in score. Doesn't fracture spanners.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    notes = staff[0][1:2]
    result = mutate(notes).split(
        [Duration(3, 64)],
        cyclic=True,
        fracture_spanners=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'32. ~
                d'32. ~
                d'32 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 3


def test_agenttools_MutationAgent_split_02():
    r'''Cyclically splits consecutive notes in score.
    Doesn't fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    result = mutate(leaves).split(
        [Duration(3, 32)],
        cyclic=True,
        fracture_spanners=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'16. ~ [ (
                c'32
                d'16 ~
                d'16 ]
            }
            {
                e'32 ~ [
                e'16.
                f'16. ~
                f'32 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 6


def test_agenttools_MutationAgent_split_03():
    r'''Cyclically splits measure in score. Doesn't fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    measures = staff[:1]
    result = mutate(measures).split(
        [Duration(3, 32)],
        cyclic=True,
        fracture_spanners=False,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/32
                c'16. [ (
            }
            {
                c'32
                d'16
            }
            {
                \time 2/32
                d'16 ]
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 3


def test_agenttools_MutationAgent_split_04():
    r'''Cyclically splits consecutive measures in score.
    Doesn't fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    measures = staff[:]
    result = mutate(measures).split(
        [Duration(3, 32)],
        cyclic=True,
        fracture_spanners=False,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/32
                c'16. [ (
            }
            {
                c'32
                d'16
            }
            {
                \time 2/32
                d'16 ]
            }
            {
                \time 1/32
                e'32 [
            }
            {
                \time 3/32
                e'16.
            }
            {
                f'16.
            }
            {
                \time 1/32
                f'32 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 6


def test_agenttools_MutationAgent_split_05():
    r'''Cyclically splits orphan measures. Doesn't fracture spanners.
    '''

    measures = [Measure((2, 8), "c'8 d'8"), Measure((2, 8), "e'8 f'8")]
    leaves = select(measures).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])

    result = mutate(measures).split(
        [Duration(3, 32)],
        cyclic=True,
        fracture_spanners=False,
        tie_split_notes=False,
        )

    music = Sequence(result).flatten()
    staff = Staff(music)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/32
                c'16. [
            }
            {
                c'32
                d'16
            }
            {
                \time 2/32
                d'16 ]
            }
            {
                \time 1/32
                e'32 [
            }
            {
                \time 3/32
                e'16.
            }
            {
                f'16.
            }
            {
                \time 1/32
                f'32 ]
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 6


def test_agenttools_MutationAgent_split_06():
    r'''Cyclically splits note in score. Doesn't fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    notes = staff[0][1:]
    result = mutate(notes).split(
        [Duration(1, 32)],
        cyclic=True,
        fracture_spanners=False,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'32 ~
                d'32 ~
                d'32 ~
                d'32 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 4


def test_agenttools_MutationAgent_split_07():
    r'''Cyclically splits consecutive notes in score.
    Doesn't fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    result = mutate(leaves).split(
        [Duration(1, 16)],
        cyclic=True,
        fracture_spanners=False,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'16 ~ [ (
                c'16
                d'16 ~
                d'16 ]
            }
            {
                e'16 ~ [
                e'16
                f'16 ~
                f'16 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 8


def test_agenttools_MutationAgent_split_08():
    r'''Cyclically splits measure in score. Doesn't fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    measures = staff[:1]
    result = mutate(measures).split(
        [Duration(1, 16)],
        cyclic=True,
        fracture_spanners=False,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/16
                c'16 ~ [ (
            }
            {
                c'16
            }
            {
                d'16 ~
            }
            {
                d'16 ]
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 4


def test_agenttools_MutationAgent_split_09():
    r'''Cyclically splits consecutive measures in score.
    Doesn't fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    measures = staff[:]
    result = mutate(measures).split(
        [Duration(3, 32)],
        cyclic=True,
        fracture_spanners=False,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/32
                c'16. ~ [ (
            }
            {
                c'32
                d'16 ~
            }
            {
                \time 2/32
                d'16 ]
            }
            {
                \time 1/32
                e'32 ~ [
            }
            {
                \time 3/32
                e'16.
            }
            {
                f'16. ~
            }
            {
                \time 1/32
                f'32 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 6


def test_agenttools_MutationAgent_split_10():
    r'''Cyclically splits note in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    notes = staff[0][1:2]
    result = mutate(notes).split(
        [Duration(3, 64)],
        cyclic=True,
        fracture_spanners=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'32. ~ ] )
                d'32. ~ [ ]
                d'32 [ ] (
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 3


def test_agenttools_MutationAgent_split_11():
    r'''Cyclically splits consecutive notes in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    result = mutate(leaves).split(
        [Duration(3, 32)],
        cyclic=True,
        fracture_spanners=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'16. ~ [ ]
                c'32 [ (
                d'16 ~ ] )
                d'16 [ ] (
            }
            {
                e'32 ~ ) [ ]
                e'16. [ (
                f'16. ~ ] )
                f'32 [ ]
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 6


def test_agenttools_MutationAgent_split_12():
    r'''Cyclically splits measure in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    measures = staff[:1]
    result = mutate(measures).split(
        [Duration(3, 32)],
        cyclic=True,
        fracture_spanners=True,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/32
                c'16. [ ]
            }
            {
                c'32 [ (
                d'16 ] )
            }
            {
                \time 2/32
                d'16 [ ] (
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 3


def test_agenttools_MutationAgent_split_13():
    r'''Cyclically splits consecutive measures in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    measures = staff[:]
    result = mutate(measures).split(
        [Duration(3, 32)],
        cyclic=True,
        fracture_spanners=True,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/32
                c'16. [ ]
            }
            {
                c'32 [ (
                d'16 ] )
            }
            {
                \time 2/32
                d'16 [ ] (
            }
            {
                \time 1/32
                e'32 ) [ ]
            }
            {
                \time 3/32
                e'16. [ ]
            }
            {
                f'16. [ ]
            }
            {
                \time 1/32
                f'32 [ ]
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 6


def test_agenttools_MutationAgent_split_14():
    r'''Cyclically splits orphan notes.
    '''

    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]

    result = mutate(notes).split(
        [Duration(3, 32)],
        cyclic=True,
        fracture_spanners=True,
        )

    music = Sequence(result).flatten()
    staff = Staff(music)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'16. ~
            c'32
            d'16 ~
            d'16
            e'32 ~
            e'16.
            f'16. ~
            f'32
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 6


def test_agenttools_MutationAgent_split_15():
    r'''Cyclically splits orphan measures. Fracture spanners.
    '''

    measures = [Measure((2, 8), "c'8 d'8"), Measure((2, 8), "e'8 f'8")]
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, measures[0][:])
    attach(beam_2, measures[1][:])

    result = mutate(measures).split(
        [Duration(3, 32)],
        cyclic=True,
        fracture_spanners=True,
        tie_split_notes=False,
        )

    music = Sequence(result).flatten()
    staff = Staff(music)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/32
                c'16. [ ]
            }
            {
                c'32 [
                d'16 ]
            }
            {
                \time 2/32
                d'16 [ ]
            }
            {
                \time 1/32
                e'32 [ ]
            }
            {
                \time 3/32
                e'16. [ ]
            }
            {
                f'16. [ ]
            }
            {
                \time 1/32
                f'32 [ ]
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 6


def test_agenttools_MutationAgent_split_16():
    r'''Cyclically splits note in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    notes = staff[0][1:]
    result = mutate(notes).split(
        [Duration(1, 32)],
        cyclic=True,
        fracture_spanners=True,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'32 ~ ] )
                d'32 ~ [ ]
                d'32 ~ [ ]
                d'32 [ ] (
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 4


def test_agenttools_MutationAgent_split_17():
    r'''Cyclically splits consecutive notes in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    result = mutate(leaves).split(
        [Duration(1, 16)],
        cyclic=True,
        fracture_spanners=True,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'16 ~ [ ]
                c'16 [ (
                d'16 ~ ] )
                d'16 [ ] (
            }
            {
                e'16 ~ ) [ ]
                e'16 [ (
                f'16 ~ ] )
                f'16 [ ]
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 8


def test_agenttools_MutationAgent_split_18():
    r'''Cyclically splits measure in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    measures = staff[:1]
    result = mutate(measures).split(
        [Duration(1, 16)],
        cyclic=True,
        fracture_spanners=True,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/16
                c'16 ~ [ ]
            }
            {
                c'16 [ ]
            }
            {
                d'16 ~ [ ]
            }
            {
                d'16 [ ] (
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 4


def test_agenttools_MutationAgent_split_19():
    r'''Cyclically splits consecutive measures in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    measures = staff[:]
    result = mutate(measures).split(
        [Duration(3, 32)],
        cyclic=True,
        fracture_spanners=True,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/32
                c'16. ~ [ ]
            }
            {
                c'32 [ (
                d'16 ~ ] )
            }
            {
                \time 2/32
                d'16 [ ] (
            }
            {
                \time 1/32
                e'32 ~ ) [ ]
            }
            {
                \time 3/32
                e'16. [ ]
            }
            {
                f'16. ~ [ ]
            }
            {
                \time 1/32
                f'32 [ ]
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 6


def test_agenttools_MutationAgent_split_20():
    r'''Force splits measure in score. Do not fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    measures = staff[:1]
    result = mutate(measures).split(
        [Duration(1, 32), Duration(3, 32), Duration(5, 32)],
        cyclic=False,
        fracture_spanners=False,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/32
                c'32 [ (
            }
            {
                \time 3/32
                c'16.
            }
            {
                \time 4/32
                d'8 ]
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 3


def test_agenttools_MutationAgent_split_21():
    r'''Force splits consecutive measures in score. Do not fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    measures = staff[:]
    result = mutate(measures).split(
        [Duration(1, 32), Duration(3, 32), Duration(5, 32)],
        cyclic=False,
        fracture_spanners=False,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/32
                c'32 [ (
            }
            {
                \time 3/32
                c'16.
            }
            {
                \time 4/32
                d'8 ]
            }
            {
                \time 1/32
                e'32 [
            }
            {
                \time 7/32
                e'16.
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 4


def test_agenttools_MutationAgent_split_22():
    r'''Force splits measure in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    measures = staff[:1]
    result = mutate(measures).split(
        [Duration(1, 32), Duration(3, 32), Duration(5, 32)],
        cyclic=False,
        fracture_spanners=True,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/32
                c'32 [ ]
            }
            {
                \time 3/32
                c'16. [ ]
            }
            {
                \time 4/32
                d'8 [ ] (
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 3


def test_agenttools_MutationAgent_split_23():
    r'''Force splits consecutive measures in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    leaves = select(staff).by_leaf()
    beam_1 = Beam()
    beam_2 = Beam()
    attach(beam_1, leaves[:2])
    attach(beam_2, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    measures = staff[:]
    result = mutate(measures).split(
        [Duration(1, 32), Duration(3, 32), Duration(5, 32)],
        cyclic=False,
        fracture_spanners=True,
        tie_split_notes=False)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 4
    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/32
                c'32 [ ]
            }
            {
                \time 3/32
                c'16. [ ]
            }
            {
                \time 4/32
                d'8 [ ] (
            }
            {
                \time 1/32
                e'32 ) [ ]
            }
            {
                \time 7/32
                e'16. [ (
                f'8 ] )
            }
        }
        '''
        ), format(staff)


def test_agenttools_MutationAgent_split_24():
    r'''Force splits orphan note. Offsets sum to less than note duration.
    '''

    note = Note("c'4")
    note = select(note)

    result = mutate(note).split(
        [(1, 32), (5, 32)],
        cyclic=False,
        fracture_spanners=True,
        tie_split_notes=False,
        )

    notes = Sequence(result).flatten()
    staff = Staff(notes)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'32
            c'8 ~
            c'32
            c'16
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 3


def test_agenttools_MutationAgent_split_25():
    r'''Force splits note in score. Fracture spanners.
    '''

    staff = Staff("c'8 [ ]")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [ ]
        }
        '''
        ), format(staff)

    notes = staff[:]
    result = mutate(notes).split(
        [Duration(1, 64), Duration(5, 64)],
        cyclic=False,
        fracture_spanners=True,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'64 [ ]
            c'16 ~ [
            c'64 ]
            c'32 [ ]
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_split_26():
    r'''Splits tuplet in score and do not fracture spanners.
    '''

    voice = Voice()
    voice.append(Tuplet((2, 3), "c'8 d'8 e'8"))
    voice.append(Tuplet((2, 3), "f'8 g'8 a'8"))
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    tuplets = voice[1:2]
    result = mutate(tuplets).split(
        [Duration(1, 12)],
        fracture_spanners=False,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \times 2/3 {
                c'8 [
                d'8
                e'8
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                f'8
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                g'8
                a'8 ]
            }
        }
        '''
        ), format(voice)

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_split_27():
    r'''Splits in-score measure with power-of-two denominator and
    do not fracture spanners.
    '''

    voice = Voice()
    voice.append(Measure((3, 8), "c'8 d'8 e'8"))
    voice.append(Measure((3, 8), "f'8 g'8 a'8"))
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    measures = voice[1:2]
    result = mutate(measures).split(
        [Duration(1, 8)],
        fracture_spanners=False,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 3/8
                c'8 [
                d'8
                e'8
            }
            {
                \time 1/8
                f'8
            }
            {
                \time 2/8
                g'8
                a'8 ]
            }
        }
        '''
        ), format(voice)

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_split_28():
    r'''Splits in-score measure without power-of-two denominator
    and do not frature spanners.
    '''

    voice = Voice()
    voice.append(Measure((3, 9), "c'8 d'8 e'8", implicit_scaling=True))
    voice.append(Measure((3, 9), "f'8 g'8 a'8", implicit_scaling=True))
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    measures = voice[1:2]
    result = mutate(measures).split(
        [Duration(1, 9)],
        fracture_spanners=False,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 3/9
                \scaleDurations #'(8 . 9) {
                    c'8 [
                    d'8
                    e'8
                }
            }
            {
                \time 1/9
                \scaleDurations #'(8 . 9) {
                    f'8
                }
            }
            {
                \time 2/9
                \scaleDurations #'(8 . 9) {
                    g'8
                    a'8 ]
                }
            }
        }
        '''
        ), format(voice)

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_split_29():
    r'''A single container can be splits in the middle.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")

    result = mutate([voice]).split(
        [Duration(1, 4)],
        fracture_spanners=False,
        )

    assert not len(voice)

    voice_1 = result[0][0]
    voice_2 = result[1][0]

    assert format(voice_1) == stringtools.normalize(
        r'''
        \new Voice {
            c'8
            d'8
        }
        '''
        ), format(voice_1)

    assert inspect_(voice_1).is_well_formed()

    assert format(voice_2) == stringtools.normalize(
        r'''
        \new Voice {
            e'8
            f'8
        }
        '''
        ), format(voice_2)

    assert inspect_(voice_2).is_well_formed()


def test_agenttools_MutationAgent_split_30():
    r'''Splits voice at negative index.
    '''

    staff = Staff([Voice("c'8 d'8 e'8 f'8")])
    voice = staff[0]

    result = mutate([voice]).split(
        [Duration(1, 4)],
        fracture_spanners=False,
        )

    left = result[0][0]
    right = result[1][0]

    assert format(left) == stringtools.normalize(
        r'''
        \new Voice {
            c'8
            d'8
        }
        '''
        ), format(left)

    assert format(right) == stringtools.normalize(
        r'''
        \new Voice {
            e'8
            f'8
        }
        '''
        ), format(right)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
        }
        '''
        ), format(voice)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \new Voice {
                c'8
                d'8
            }
            \new Voice {
                e'8
                f'8
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_split_31():
    r'''Slpit container in score and do not fracture spanners.
    '''

    staff = Staff([Container("c'8 d'8 e'8 f'8")])
    voice = staff[0]
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    result = mutate([voice]).split(
        [Duration(1, 4)],
        fracture_spanners=False,
        )

    left = result[0][0]
    right = result[1][0]

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        ), format(staff)

    assert format(left) == stringtools.normalize(
        r'''
        {
            c'8 [
            d'8
        }
        '''
        ), format(left)

    assert format(right) == stringtools.normalize(
        r'''
        {
            e'8
            f'8 ]
        }
        '''
        ), format(right)

    assert format(voice) == stringtools.normalize(
        r'''
        {
        }
        '''
        ), format(voice)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_split_32():
    r'''Splits tuplet in score and do not fracture spanners.
    '''

    tuplet = Tuplet((4, 5), "c'8 c'8 c'8 c'8 c'8")
    voice = Voice([tuplet])
    staff = Staff([voice])
    beam = Beam()
    attach(beam, tuplet[:])

    result = mutate([tuplet]).split(
        [Duration(1, 5)],
        fracture_spanners=False,
        )

    left = result[0][0]
    right = result[1][0]

    assert format(left) == stringtools.normalize(
        r'''
        \tweak edge-height #'(0.7 . 0)
        \times 4/5 {
            c'8 [
            c'8
        }
        '''
        ), format(left)

    assert format(right) == stringtools.normalize(
        r'''
        \tweak edge-height #'(0.7 . 0)
        \times 4/5 {
            c'8
            c'8
            c'8 ]
        }
        '''
        ), format(right)

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 4/5 {
        }
        '''
        ), format(tuplet)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \tweak edge-height #'(0.7 . 0)
            \times 4/5 {
                c'8 [
                c'8
            }
            \tweak edge-height #'(0.7 . 0)
            \times 4/5 {
                c'8
                c'8
                c'8 ]
            }
        }
        '''
        ), format(voice)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \new Voice {
                \tweak edge-height #'(0.7 . 0)
                \times 4/5 {
                    c'8 [
                    c'8
                }
                \tweak edge-height #'(0.7 . 0)
                \times 4/5 {
                    c'8
                    c'8
                    c'8 ]
                }
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_split_33():
    r'''Splits triplet, and fracture spanners.
    '''

    voice = Voice()
    voice.append(Tuplet((2, 3), "c'8 d'8 e'8"))
    voice.append(Tuplet((2, 3), "f'8 g'8 a'8"))
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    tuplet = voice[1]

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \times 2/3 {
                c'8 [
                d'8
                e'8
            }
            \times 2/3 {
                f'8
                g'8
                a'8 ]
            }
        }
        '''
        ), format(voice)

    result = mutate([tuplet]).split(
        [Duration(1, 12)],
        fracture_spanners=True,
        )

    left = result[0][0]
    right = result[1][0]

    assert format(left) == stringtools.normalize(
        r'''
        \tweak edge-height #'(0.7 . 0)
        \times 2/3 {
            f'8 ]
        }
        '''
        ), format(left)

    assert format(right) == stringtools.normalize(
        r'''
        \tweak edge-height #'(0.7 . 0)
        \times 2/3 {
            g'8 [
            a'8 ]
        }
        '''
        ), format(right)

    assert format(tuplet) == '\\times 2/3 {\n}'

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \times 2/3 {
                c'8 [
                d'8
                e'8
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                f'8 ]
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                g'8 [
                a'8 ]
            }
        }
        '''
        ), format(voice)

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_split_34():
    r'''Splits measure with power-of-two time signature denominator.
    Fracture spanners.
    '''

    voice = Voice()
    voice.append(Measure((3, 8), "c'8 d'8 e'8"))
    voice.append(Measure((3, 8), "f'8 g'8 a'8"))
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    measure = voice[1]

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 3/8
                c'8 [
                d'8
                e'8
            }
            {
                f'8
                g'8
                a'8 ]
            }
        }
        '''
        ), format(voice)

    result = mutate([measure]).split(
        [Duration(1, 8)],
        fracture_spanners=True,
        )

    left = result[0][0]
    right = result[1][0]

    assert format(left) == stringtools.normalize(
        r'''
        {
            \time 1/8
            f'8 ]
        }
        '''
        ), format(left)

    assert format(right) == stringtools.normalize(
        r'''
        {
            \time 2/8
            g'8 [
            a'8 ]
        }
        '''
        ), format(right)

    assert pytest.raises(UnderfullContainerError, 'format(measure)')

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 3/8
                c'8 [
                d'8
                e'8
            }
            {
                \time 1/8
                f'8 ]
            }
            {
                \time 2/8
                g'8 [
                a'8 ]
            }
        }
        '''
        ), format(voice)

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_split_35():
    r'''Splits measure without power-of-two time signature denominator.
    Fractures spanners.
    '''

    voice = Voice()
    measure = Measure((3, 9), "c'8 d'8 e'8", implicit_scaling=True)
    voice.append(measure)
    measure = Measure((3, 9), "f'8 g'8 a'8", implicit_scaling=True)
    voice.append(measure)
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    measure = voice[1]

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 3/9
                \scaleDurations #'(8 . 9) {
                    c'8 [
                    d'8
                    e'8
                }
            }
            {
                \scaleDurations #'(8 . 9) {
                    f'8
                    g'8
                    a'8 ]
                }
            }
        }
        '''
        ), format(voice)

    result = mutate([measure]).split(
        [Duration(1, 9)],
        fracture_spanners=True,
        )

    left = result[0][0]
    right = result[1][0]

    assert format(left) == stringtools.normalize(
        r'''
        {
            \time 1/9
            \scaleDurations #'(8 . 9) {
                f'8 ]
            }
        }
        '''
        ), format(left)

    assert format(right) == stringtools.normalize(
        r'''
        {
            \time 2/9
            \scaleDurations #'(8 . 9) {
                g'8 [
                a'8 ]
            }
        }
        '''
        ), format(right)

    assert pytest.raises(UnderfullContainerError, 'format(measure)')

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                \time 3/9
                \scaleDurations #'(8 . 9) {
                    c'8 [
                    d'8
                    e'8
                }
            }
            {
                \time 1/9
                \scaleDurations #'(8 . 9) {
                    f'8 ]
                }
            }
            {
                \time 2/9
                \scaleDurations #'(8 . 9) {
                    g'8 [
                    a'8 ]
                }
            }
        }
        '''
        ), format(voice)

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_split_36():
    r'''Splits voice outside of score.
    Fracture spanners.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8
            f'8 ]
        }
        '''
        ), format(voice)

    result = mutate([voice]).split(
        [Duration(1, 4)],
        fracture_spanners=True,
        )

    left = result[0][0]
    right = result[1][0]

    assert format(left) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ]
        }
        '''
        ), format(left)

    assert format(right) == stringtools.normalize(
        r'''
        \new Voice {
            e'8 [
            f'8 ]
        }
        '''
        ), format(right)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
        }
        '''
        ), format(voice)


def test_agenttools_MutationAgent_split_37():
    r'''Splits measure in score and fracture spanners.
    '''

    staff = Staff()
    staff.append(Measure((2, 8), "c'8 d'8"))
    staff.append(Measure((2, 8), "e'8 f'8"))
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[-2:])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    measures = staff[:1]
    result = mutate(measures).split(
        [Duration(1, 8)],
        fracture_spanners=True,
        )

    left = result[0][0]
    right = result[1][0]

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/8
                c'8 [ ]
            }
            {
                d'8 [ ] (
            }
            {
                \time 2/8
                e'8 [
                f'8 ] )
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_split_38():
    r'''Splits in-score measure with power-of-two time signature denominator.
    Fractured spanners but do not tie over splits locus.
    Measure contents necessitate denominator change.
    '''

    staff = Staff([Measure((3, 8), "c'8. d'8.")])
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/8
                c'8. [ (
                d'8. ] )
            }
        }
        '''
        ), format(staff)

    measures = staff[:1]
    result = mutate(measures).split(
        [Duration(3, 16)],
        fracture_spanners=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/16
                c'8. [ ]
            }
            {
                d'8. [ ]
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 2


def test_agenttools_MutationAgent_split_39():
    r'''Splits cyclic.
    Leave spanner attaching to container contents untouched.
    '''

    voice = Voice([Container("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")])
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
                d'8
                e'8
                f'8
                g'8
                a'8
                b'8
                c''8 ] )
            }
        }
        '''
        ), format(voice)

    note = voice[0]
    result = mutate(note).split(
        [Duration(1, 8), Duration(3, 8)],
        cyclic=True,
        fracture_spanners=False,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
            }
            {
                d'8
                e'8
                f'8
            }
            {
                g'8
            }
            {
                a'8
                b'8
                c''8 ] )
            }
        }
        '''
        ), format(voice)

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_split_40():
    r'''Cyclic 1 splits all elements in container.
    '''

    voice = Voice([Container("c'8 d'8 e'8 f'8")])
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
                d'8
                e'8
                f'8 ] )
            }
        }
        '''
        ), format(voice)

    container = voice[0]
    result = mutate(container).split(
        [Duration(1, 8)],
        cyclic=True,
        fracture_spanners=False,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
            }
            {
                d'8
            }
            {
                e'8
            }
            {
                f'8 ] )
            }
        }
        '''
        ), format(voice)

    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_split_41():
    r'''Splits cyclic.
    Fracture spanners attaching directly to container.
    Leave spanner attaching to container contents untouched.
    '''

    voice = Voice([Container("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")])
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
                d'8
                e'8
                f'8
                g'8
                a'8
                b'8
                c''8 ] )
            }
        }
        '''
        ), format(voice)

    container = voice[0]
    result = mutate(container).split(
        [Duration(1, 8), Duration(3, 8)],
        cyclic=True,
        fracture_spanners=True,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ ]
            }
            {
                d'8 [ (
                e'8
                f'8 ] )
            }
            {
                g'8 [ ]
            }
            {
                a'8 [ (
                b'8
                c''8 ] )
            }
        }
        '''
        ), format(voice)

    assert inspect_(voice).is_well_formed()
    assert len(result) == 4


def test_agenttools_MutationAgent_split_42():
    r'''Cyclic by 1 splits all elements in container.
    '''

    voice = Voice([Container("c'8 d'8 e'8 f'8")])
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
                d'8
                e'8
                f'8 ] )
            }
        }
        '''
        ), format(voice)

    container = voice[0]
    result = mutate(container).split(
        [Duration(1, 8)],
        cyclic=True,
        fracture_spanners=True,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ ]
            }
            {
                d'8 [ ]
            }
            {
                e'8 [ ]
            }
            {
                f'8 [ ]
            }
        }
        '''
        ), format(voice)

    assert inspect_(voice).is_well_formed()
    assert len(result) == 4


def test_agenttools_MutationAgent_split_43():
    r'''Extra durations are ignored.
    Result contains no empty shards.
    '''

    voice = Voice([Container("c'8 d'8 e'8 f'8")])
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
                d'8
                e'8
                f'8 ] )
            }
        }
        '''
        ), format(voice)

    container = voice[0]
    result = mutate(container).split(
        5 * [Duration(2, 8)],
        cyclic=True,
        fracture_spanners=True,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
                d'8 ] )
            }
            {
                e'8 [ (
                f'8 ] )
            }
        }
        '''
        ), format(voice)

    assert inspect_(voice).is_well_formed()
    assert len(result) == 2


def test_agenttools_MutationAgent_split_44():
    r'''Empty durations list.
    Expression remains unaltered.
    '''

    voice = Voice([Container("c'8 d'8 e'8 f'8")])
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
                d'8
                e'8
                f'8 ] )
            }
        }
        '''
        ), format(voice)

    container = voice[0]
    result = mutate(container).split(
        [],
        cyclic=True,
        fracture_spanners=True,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
                d'8
                e'8
                f'8 ] )
            }
        }
        '''
        ), format(voice)

    assert inspect_(voice).is_well_formed()
    assert len(result) == 1


def test_agenttools_MutationAgent_split_45():
    r'''Splits one time.
    Fracture spanners attaching directly to container.
    Leave spanner attaching to container contents untouched.
    '''

    voice = Voice([Container("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")])
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
                d'8
                e'8
                f'8
                g'8
                a'8
                b'8
                c''8 ] )
            }
        }
        '''
        ), format(voice)

    container = voice[0]
    result = mutate(container).split(
        [Duration(1, 8), Duration(3, 8)],
        cyclic=False,
        fracture_spanners=False,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
            }
            {
                d'8
                e'8
                f'8
            }
            {
                g'8
                a'8
                b'8
                c''8 ] )
            }
        }
        '''
        ), format(voice)

    assert inspect_(voice).is_well_formed()
    assert len(result) == 3


def test_agenttools_MutationAgent_split_46():
    r'''Splits one time.
    Fracture spanners attaching directly to container.
    Leave spanner attaching to container contents untouched.
    '''

    voice = Voice([Container("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")])
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
                d'8
                e'8
                f'8
                g'8
                a'8
                b'8
                c''8 ] )
            }
        }
        '''
        ), format(voice)

    container = voice[0]
    result = mutate(container).split(
        [Duration(1, 8), Duration(3, 8)],
        cyclic=False,
        fracture_spanners=True,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ ]
            }
            {
                d'8 [ (
                e'8
                f'8 ] )
            }
            {
                g'8 [ (
                a'8
                b'8
                c''8 ] )
            }
        }
        '''
        ), format(voice)

    assert inspect_(voice).is_well_formed()
    assert len(result) == 3


def test_agenttools_MutationAgent_split_47():
    r'''Extra durations are ignored.
    Result contains no empty shards.
    '''

    voice = Voice([Container("c'8 d'8 e'8 f'8")])
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    slur = Slur()
    attach(slur, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
                d'8
                e'8
                f'8 ] )
            }
        }
        '''
        ), format(voice)

    container = voice[0]
    result = mutate(container).split(
        5 * [Duration(2, 8)],
        cyclic=False,
        fracture_spanners=True,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
                d'8 ] )
            }
            {
                e'8 [ (
                f'8 ] )
            }
        }
        '''
        ), format(voice)

    assert inspect_(voice).is_well_formed()
    assert len(result) == 2


def test_agenttools_MutationAgent_split_48():
    r'''Splits leaf at relative offset that is both non-assignable
    and non-power-of-two.
    '''

    staff = Staff("c'4")

    notes = staff[:1]
    result = mutate(notes).split(
        [Duration(5, 24)],
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                c'4 ~
                c'16 ~
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                c'16
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()


# container._split_at_index() works here;
# scoretools.split() doesn't work here.
# eventually make scoretools.split() work here.
def test_agenttools_MutationAgent_split_49():
    r'''Splits in-score measure without power-of-two time
    signature denominator. Fractured spanners but do not tie
    over splits locus. Measure contents necessitate denominator change.
    '''
    pytest.skip('TODO: make this work.')

    staff = Staff([Measure((3, 12), "c'8. d'8.")])
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, staff[0])
    slur = Slur()
    attach(slur, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/12
                \scaleDurations #'(2 . 3) {
                    c'8. [ (
                    d'8. ] )
                }
            }
        }
        '''
        ), format(staff)

    measures = staff[:1]
    result = mutate(measures).split(
        [Duration(3, 24)],
        fracture_spanners=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/24
                \scaleDurations #'(2 . 3) {
                    c'8. [ ] (
                }
            }
            {
                \scaleDurations #'(2 . 3) {
                    d'8. ) [ ]
                }
            }
        }
        '''
        ), format(staff)

    assert inspect_(staff).is_well_formed()
    assert len(result) == 2



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_agenttools_MutationAgent_replace_measure_contents_01():
    r'''Contents duration less than sum of duration of measures.
    Note spacer skip at end of second measure.
    '''

    measures = scoretools.make_spacer_skip_measures([(1, 8), (3, 16)])
    staff = Staff(measures)
    notes = [Note("c'16"), Note("d'16"), Note("e'16"), Note("f'16")]
    mutate(staff).replace_measure_contents(notes)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/8
                c'16
                d'16
            }
            {
                \time 3/16
                e'16
                f'16
                s1 * 1/16
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_replace_measure_contents_02():
    r'''Some contents too big for some measures.
    Small measures skipped.
    '''

    time_signatures = [(1, 16), (3, 16), (1, 16), (3, 16)]
    measures = scoretools.make_spacer_skip_measures(time_signatures)
    staff = Staff(measures)
    notes = [Note("c'8"), Note("d'8")]
    mutate(staff).replace_measure_contents(notes)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/16
                s1 * 1/16
            }
            {
                \time 3/16
                c'8
                s1 * 1/16
            }
            {
                \time 1/16
                s1 * 1/16
            }
            {
                \time 3/16
                d'8
                s1 * 1/16
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_replace_measure_contents_03():
    r'''Raise MissingMeasureError when input expression
    contains no measures.
    '''

    note = Note("c'4")
    notes = [Note("c'8"), Note("d'8")]
    statement = 'mutate(note).replace_measure_contents(notes)'
    assert pytest.raises(MissingMeasureError, statement)


def test_agenttools_MutationAgent_replace_measure_contents_04():
    r'''Raise StopIteration when not enough measures.
    '''

    measures = scoretools.make_spacer_skip_measures([(1, 8), (1, 8)])
    staff = Staff(measures)
    notes = [Note("c'16"), Note("d'16"), Note("e'16"),
        Note("f'16"), Note("g'16"), Note("a'16")]
    statement = 'mutate(staff).replace_measure_contents(notes)'
    assert pytest.raises(StopIteration, statement)


def test_agenttools_MutationAgent_replace_measure_contents_05():
    r'''Populate measures even when not enough total measures.
    '''

    measures = scoretools.make_spacer_skip_measures([(1, 8), (1, 8)])
    staff = Staff(measures)
    notes = [Note("c'16"), Note("d'16"), Note("e'16"),
        Note("f'16"), Note("g'16"), Note("a'16")]

    try:
        mutate(staff).replace_measure_contents(notes)
    except StopIteration:
        pass

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/8
                c'16
                d'16
            }
            {
                e'16
                f'16
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_agenttools_MutationAgent_replace_measure_contents_06():
    r'''Preserve ties.
    '''

    maker = rhythmmakertools.NoteRhythmMaker()
    durations = [(5, 16), (3, 16)]
    leaf_lists = maker(durations)
    leaves = Sequence(leaf_lists).flatten()
    measures = scoretools.make_spacer_skip_measures(durations)
    staff = Staff(measures)
    measures = mutate(staff).replace_measure_contents(leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 5/16
                c'4 ~
                c'16
            }
            {
                \time 3/16
                c'8.
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
import collections
import types


def list_all_functions(modules=None):
    r'''Lists all public functions defined in `modules`.

    ::

        >>> all_functions = documentationtools.list_all_functions(
        ...     modules='abjad',
        ...     )

    '''
    from abjad import abjad_configuration
    from abjad.tools import documentationtools
    all_functions = set()
    paths = []
    if modules is None:
        paths.append(abjad_configuration.abjad_directory)
    elif isinstance(modules, types.ModuleType):
        paths.extend(modules.__path__)
    elif isinstance(modules, collections.Iterable):
        for module in modules:
            if isinstance(module, types.ModuleType):
                paths.extend(module.__path__)
    for path in paths:
        for x in documentationtools.yield_all_classes(code_root=path):
            all_functions.add(x)
    return list(sorted(
        all_functions,
        key=lambda x: (x.__module__, x.__name__)
        ))



# -*- coding: utf-8 -*-
import collections
from abjad.tools import systemtools
from abjad.tools.datastructuretools.TypedCollection import TypedCollection


class TypedList(TypedCollection):
    r'''A typed list.

    ..  container:: example

        No item coercion:

        ::

            >>> list_ = datastructuretools.TypedList()
            >>> list_.append(23)
            >>> list_.append('foo')
            >>> list_.append(False)
            >>> list_.append((1, 2, 3))
            >>> list_.append(3.14159)

        ::

            >>> f(list_)
            datastructuretools.TypedList(
                [
                    23,
                    'foo',
                    False,
                    (1, 2, 3),
                    3.14159,
                    ]
                )

    ..  container:: example

        Named pitch item coercion:

        ::

            >>> pitch_list = datastructuretools.TypedList(
            ...     item_class=NamedPitch,
            ...     )
            >>> pitch_list.append(0)
            >>> pitch_list.append("d'")
            >>> pitch_list.append(('e', 4))
            >>> pitch_list.append(NamedPitch("f'"))

        ::

            >>> f(pitch_list)
            datastructuretools.TypedList(
                [
                    pitchtools.NamedPitch("c'"),
                    pitchtools.NamedPitch("d'"),
                    pitchtools.NamedPitch("e'"),
                    pitchtools.NamedPitch("f'"),
                    ],
                item_class=pitchtools.NamedPitch,
                )

    Ordered collection with optional item coercion.

    Implements the list interface.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_keep_sorted',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        items=None,
        item_class=None,
        keep_sorted=False,
        ):
        TypedCollection.__init__(
            self,
            item_class=item_class,
            items=items,
            )
        self._collection = []
        if keep_sorted is not None:
            assert isinstance(keep_sorted, bool), repr(keep_sorted)
        self._keep_sorted = keep_sorted
        items = items or []
        the_items = []
        for item in items:
            the_items.append(self._item_coercer(item))
        self.extend(the_items)

    ### SPECIAL METHODS ###

    def __delitem__(self, i):
        r'''Aliases list.__delitem__().

        Returns none.
        '''
        self._on_removal(self._collection[i])
        del(self._collection[i])

    def __getitem__(self, argument):
        r'''Gets item or slice identified  by `argument`.

        Returns item or slice.
        '''
        return self._collection.__getitem__(argument)

    def __iadd__(self, argument):
        r'''Adds `argument` in place to typed list.

        ..  container:: example

            ::

                >>> dynamic_list = datastructuretools.TypedList(item_class=Dynamic)
                >>> dynamic_list.append('ppp')
                >>> dynamic_list += ['p', 'mp', 'mf', 'fff']

            ::

                >>> f(dynamic_list)
                datastructuretools.TypedList(
                    [
                        indicatortools.Dynamic(
                            name='ppp',
                            ),
                        indicatortools.Dynamic(
                            name='p',
                            ),
                        indicatortools.Dynamic(
                            name='mp',
                            ),
                        indicatortools.Dynamic(
                            name='mf',
                            ),
                        indicatortools.Dynamic(
                            name='fff',
                            ),
                        ],
                    item_class=indicatortools.Dynamic,
                    )

        Returns typed list.
        '''
        self.extend(argument)
        return self

    def __reversed__(self):
        r'''Aliases list.__reversed__().

        Returns generator.
        '''
        return self._collection.__reversed__()

    def __setitem__(self, i, argument):
        r'''Sets item `i` equal to `argument`.

        ..  container:: example

            Sets item:

            ::

                >>> pitch_list = datastructuretools.TypedList(
                ...     item_class=NamedPitch,
                ...     )
                >>> pitch_list.append(0)
                >>> pitch_list.append("d'")
                >>> pitch_list.append(('e', 4))
                >>> pitch_list.append(NamedPitch("f'"))

            ::

                >>> pitch_list[-1] = 'gqs,'
                >>> f(pitch_list)
                datastructuretools.TypedList(
                    [
                        pitchtools.NamedPitch("c'"),
                        pitchtools.NamedPitch("d'"),
                        pitchtools.NamedPitch("e'"),
                        pitchtools.NamedPitch('gqs,'),
                        ],
                    item_class=pitchtools.NamedPitch,
                    )

        ..  container:: example

            Sets slice:

            ::

                >>> pitch_list[-1:] = ["f'", "g'", "a'", "b'", "c''"]
                >>> f(pitch_list)
                datastructuretools.TypedList(
                    [
                        pitchtools.NamedPitch("c'"),
                        pitchtools.NamedPitch("d'"),
                        pitchtools.NamedPitch("e'"),
                        pitchtools.NamedPitch("f'"),
                        pitchtools.NamedPitch("g'"),
                        pitchtools.NamedPitch("a'"),
                        pitchtools.NamedPitch("b'"),
                        pitchtools.NamedPitch("c''"),
                        ],
                    item_class=pitchtools.NamedPitch,
                    )

        Returns none.
        '''
        if isinstance(i, int):
            new_item = self._item_coercer(argument)
            old_item = self._collection[i]
            self._on_removal(old_item)
            self._on_insertion(new_item)
            self._collection[i] = new_item
        elif isinstance(i, slice):
            new_items = [self._item_coercer(item) for item in argument]
            old_items = self._collection[i]
            for old_item in old_items:
                self._on_removal(old_item)
            for new_item in new_items:
                self._on_insertion(new_item)
            self._collection[i] = new_items
        if self.keep_sorted:
            self.sort()

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        agent = systemtools.StorageFormatAgent(self)
        names = list(agent.signature_keyword_names)
        if 'items' in names:
            names.remove('items')
        if 'keep_sorted' in names:
            names.remove('keep_sorted')
        return systemtools.FormatSpecification(
            self,
            repr_is_indented=False,
            storage_format_args_values=[self._collection],
            storage_format_kwargs_names=names,
            )

    ### PUBLIC METHODS ###

    def append(self, item):
        r'''Appends `item` to typed list.

        ..  container:: example

            ::

                >>> integer_list = datastructuretools.TypedList(item_class=int)
                >>> integer_list.append('1')
                >>> integer_list.append(2)
                >>> integer_list.append(3.4)
                >>> integer_list[:]
                [1, 2, 3]

        Returns none.
        '''
        item = self._item_coercer(item)
        self._on_insertion(item)
        self._collection.append(item)
        if self.keep_sorted:
            self.sort()

    def count(self, item):
        r'''Gets count of `item` in typed list.

        ..  container:: example

            ::

                >>> integer_list = datastructuretools.TypedList(item_class=int)
                >>> integer_list.extend([0, 0., '0', 99])

            ::

                >>> integer_list.count(0)
                3
                >>> integer_list.count(1)
                0
                >>> integer_list.count(99)
                1

        Returns nonnegative integer.
        '''
        item = self._item_coercer(item)
        return self._collection.count(item)

    def extend(self, items):
        r'''Extends typed list with `items`.
        
        ..  container:: example

            ::

                >>> integer_list = datastructuretools.TypedList(item_class=int)
                >>> integer_list.extend(['0', 1.0, 2, 3.14159])
                >>> integer_list
                TypedList([0, 1, 2, 3], item_class=int)

        Returns none.
        '''
        for item in items:
            self.append(item)
        if self.keep_sorted:
            self.sort()

    def index(self, item):
        r'''Gets index of `item` in typed list.

        ..  container:: example

            ::

                >>> pitch_list = datastructuretools.TypedList(
                ...     item_class=NamedPitch,
                ...     )
                >>> pitch_list.extend(['cqf', "as'", 'b,', 'dss'])

            ::

                >>> pitch_list.index(NamedPitch('cqf'))
                0
                >>> pitch_list.index(NamedPitch("as'"))
                1
                >>> pitch_list.index('b,')
                2
                >>> pitch_list.index('dss')
                3

        Returns nonnegative integer.
        '''
        item = self._item_coercer(item)
        return self._collection.index(item)

    def insert(self, i, item):
        r'''Insert `item` into typed list.

        ..  container:: example

            Inserts into typed list.

            ::

                >>> integer_list = datastructuretools.TypedList(item_class=int)
                >>> integer_list.extend(['1', 2, 4.3])
                >>> integer_list
                TypedList([1, 2, 4], item_class=int)

            ::

                >>> integer_list.insert(0, '0')
                >>> integer_list
                TypedList([0, 1, 2, 4], item_class=int)

            ::

                >>> integer_list.insert(1, '9')
                >>> integer_list
                TypedList([0, 9, 1, 2, 4], item_class=int)

        Returns none.
        '''
        item = self._item_coercer(item)
        self._on_insertion(item)
        result = self._collection.insert(i, item)
        if self.keep_sorted:
            self.sort()
        return result

    def pop(self, i=-1):
        r'''Pops item `i` from typed list.

        Returns item.
        '''
        result = self._collection.pop(i)
        self._on_removal(result)
        if self.keep_sorted:
            self.sort()
        return result

    def remove(self, item):
        r'''Removes `item` from typed list.

        ..  container:: example

            ::

                >>> integer_list = datastructuretools.TypedList(item_class=int)
                >>> integer_list.extend(('0', 1.0, 2, 3.14159))
                >>> integer_list[:]
                [0, 1, 2, 3]

            ::

                >>> integer_list.remove('1')
                >>> integer_list[:]
                [0, 2, 3]

        Returns none.
        '''
        item = self._item_coercer(item)
        index = self._collection.index(item)
        item = self._collection[index]
        self._on_removal(item)
        del(self._collection[index])
        if self.keep_sorted:
            self.sort()

    def reverse(self):
        r'''Reverses items in typed list.
        '''
        self._collection.reverse()

    def sort(self, cmp=None, key=None, reverse=False):
        r'''Sorts items in typed list.
        '''
        if cmp is not None:
            def cmp_to_key(comparator):
                r'''Convert a `cmp` function into a `key` function for use
                with `sort()`.
                '''
                class CmpToKey(object):

                    def __init__(self, argument):
                        self.argument = argument

                    def __lt__(self, other):
                        return comparator(self.argument, other.argument) < 0

                    def __gt__(self, other):
                        return comparator(self.argument, other.argument) > 0

                    def __eq__(self, other):
                        return comparator(self.argument, other.argument) == 0

                    def __le__(self, other):
                        return comparator(self.argument, other.argument) <= 0

                    def __ge__(self, other):
                        return comparator(self.argument, other.argument) >= 0

                    def __ne__(self, other):
                        return comparator(self.argument, other.argument) != 0

                return CmpToKey
            key = cmp_to_key(cmp)
        self._collection.sort(key=key, reverse=reverse)

    ### PUBLIC PROPERTIES ###

    @property
    def keep_sorted(self):
        r'''Is true when typed list keeps items sorted. Otherwise false.

        Defaults to none.

        Set to true, false or none.

        Returns true, false or none.
        '''
        return self._keep_sorted

    @keep_sorted.setter
    def keep_sorted(self, argument):
        assert isinstance(argument, (bool, type(None)))
        self._keep_sorted = argument


collections.MutableSequence.register(TypedList)



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_ComplexBeam___init___01():
    r'''Initialize empty complex beam spanner.
    '''

    beam = spannertools.ComplexBeam()
    assert isinstance(beam, spannertools.ComplexBeam)


def test_spannertools_ComplexBeam___init___02():

    staff = Staff("c'16 e'16 r16 f'16 g'2")
    beam = spannertools.ComplexBeam()
    attach(beam, staff[:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set stemLeftBeamCount = #0
            \set stemRightBeamCount = #2
            c'16 [
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #2
            e'16 ]
            r16
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #0
            f'16 [ ]
            g'2
        }
        '''
        )



# -*- coding: utf-8 -*-
import os
from abjad import *
configuration = systemtools.AbjadConfiguration()

path_1 = os.path.join(
    configuration.abjad_directory,
    'test_1.pdf',
    )
path_2 = os.path.join(
    configuration.abjad_directory,
    'test_2.pdf',
    )


def test_systemtools_TestManager__compare_pdfs_01():
    r'''Is true when PDFs are the same.
    '''

    with systemtools.FilesystemState(remove=[path_1]):
        note = Note("c'4")
        persist(note).as_pdf(path_1, remove_ly=True)
        assert systemtools.TestManager._compare_pdfs(path_1, path_1)


def test_systemtools_TestManager__compare_pdfs_02():
    r'''Is true when PDFs differ but contain the same music.
    '''

    with systemtools.FilesystemState(remove=[path_1, path_2]):
        note = Note("c'4")
        persist(note).as_pdf(path_1, remove_ly=True)
        note = Note("c'4")
        persist(note).as_pdf(path_2, remove_ly=True)
        assert systemtools.TestManager._compare_pdfs(path_1, path_2)


def test_systemtools_TestManager__compare_pdfs_03():
    r'''False when PDFs contain different music.
    '''

    with systemtools.FilesystemState(remove=[path_1, path_2]):
        note = Note("c'4")
        persist(note).as_pdf(path_1, remove_ly=True)
        note = Note("d'4")
        persist(note).as_pdf(path_2, remove_ly=True)
        assert not systemtools.TestManager._compare_pdfs(path_1, path_2)
